#!/usr/bin/env python
# -*- coding: utf-8 -*-

import connection_utils
import subprocess

class ConnectionEditor:
    def __init__(self, d, wvDialConf, wvDialConfTmp):
        self.d = d
        self.wvDialConf = wvDialConf
        self.wvDialConfTmp = wvDialConfTmp
        self.wvDialProcess = None
        self.confLoaded = False
        self.config = None
        self.pingCount = "4"
        self.pingAddress = ""
        self.isExit = False

    def loadConf(self):
        self.config = connection_utils.loadWvDialConfig(self.wvDialConfTmp)
        if not self.config:
            self.d.msgbox("Не удалось загрузить файл конфигурации - " + self.wvDialConfTmp)
            self.close()


    def mainMenu(self):
        """Главное меню"""
        return self.d.menu(
            "Ручная настройка соединения",
            width=40, height=12, menu_height=5,
            choices=[("edit", "Изменить настройки"),
                     ("connect", "Тестовое подключение"),
                     ("ping", "Проверка соединения"),
                     ("save", "Сохранить изменения"),
                     ("quit", "Выйти")]
            )

    def editMenu(self):
        """Меню редактирования параметров"""
        (code, tag) = self.d.form (
            title = "Редактор параметров",
            text = "Параметры соединения:",
            height=12, width=54, form_height=6,
            fields = [(u"Порт:", self.config['port'], 60),
                      (u"Скорость:", self.config['baudrate'], 60),
                      (u"Номер:", self.config['phone'], 60),
                      (u"Точка доступа:", self.config['apn'], 60),
                      (u"Имя пользователя:", self.config['username'], 60),
                      (u"Пароль:", self.config['password'], 60)]
            )
        if code == self.d.DIALOG_OK:
            self.config['port'] = tag[0]
            self.config['baudrate'] = tag[1]
            self.config['phone'] = tag[2]
            self.config['apn'] = tag[3]
            self.config['username'] = tag[4]
            self.config['password'] = tag[5]
            
        if not connection_utils.saveWvDialConfig(self.wvDialConfTmp, self.config):
            self.d.msgbox("Не удалось сохранить файл конфигурации - " + self.wvDialConfTmp, height=6)
            self.isExit = true
 
            
    def run(self):
        self.loadConf()
        try:
            self.isExit = False
            while not self.isExit:
                (code, tag) = self.mainMenu()
                if code == self.d.DIALOG_OK:
                    if tag == "edit":
                        self.editMenu()
                    elif tag == "connect":
                        if self.wvDialProcess:
                            self.wvDialProcess.terminate()
                            self.wvDialProcess = None
                        (status, wvdialProcess) = connection_utils.testConnectionDialog(self.d, self.wvDialConfTmp)
                        self.wvDialProcess = wvdialProcess
                    elif tag == "ping":
                        self.pingAddress, self.pingCount = connection_utils.pingDialog(self.d, self.pingAddress, self.pingCount)
                    elif tag == "save":
                        self.isExit = connection_utils.moveWvDialConfig(self.d, self.wvDialConf, self.wvDialConfTmp)  
                    else:
                        self.isExit = True        
                else:
                    self.isExit = True
        finally:
            if self.wvDialProcess:
                self.wvDialProcess.terminate()
                self.wvDialProcess = None
           