#!/usr/bin/env python
# -*- coding: utf-8 -*-

import subprocess
import ConfigParser
import re
import time
import os

DIALOG_OK_LABEL = "Далее"
DIALOG_EXTRA_LABEL = "Назад"
DIALOG_CANCEL_LABEL = "Отмена"
DIALOG_SKIP_LABEL = "Пропустить"

providers = {'beeline': {'title' : "Билайн",
                         'phone' : "*99#",
                         'password' : "beeline",
                         'username' : "beeline",
                         'apn' : "internet.beeline.ru" 
                        },
             'mts':     {'title' : "МТС",
                         'phone' : "*99#",
                         'password' : "mts",
                         'username' : "mts",
                         'apn' : "internet.mts.ru"
                        },
             'megafon': {'title' : "Мегафон",
                         'phone' : "*99#",
                         'password' : "",
                         'username' : "",
                         'apn' : "internet"
                        }
             }

baudrates = ["7200", "9600", "14400", "28800", "38400", "57600", "115200", "230400", "460800"]

def setProvider(config, provider):
    config['phone'] = providers[provider]['phone']
    config['password'] = providers[provider]['password']
    config['username'] = providers[provider]['username']
    config['apn'] = providers[provider]['apn']

def getConfigValue(conf, section, name, defaultValue):
    return conf.get(section, name) if conf.has_option(section, name) else defaultValue    

def loadWvDialConfig(confFile):
    config = ConfigParser.ConfigParser()
    config.read(confFile)
    section = "Dialer Defaults";
    if not config.has_section(section):
        return None
    init2 = getConfigValue(config, section, "Init2", None)
    apn = re.search("AT\+CGDCONT=1,\"IP\",\"(.*)\"", init2).group(1) if init2 else ""
    return {'port' : getConfigValue(config, section, "modem", ""),
            'baudrate' : getConfigValue(config, section, "baud", ""),
            'phone' : getConfigValue(config, section, "phone", ""),
            'apn' : apn,
            'password' : getConfigValue(config, section, "password", ""),
            'username' : getConfigValue(config, section, "username", "")
            }

def saveWvDialConfig(confFile, config):
    confParser = ConfigParser.ConfigParser()
    confParser.read(confFile)
    section = "Dialer Defaults";
    if not confParser.has_section(section):
        return False
    confParser.set(section, "modem", config['port'])
    confParser.set(section, 'baud', config['baudrate'])
    confParser.set(section, 'phone', config['phone'])
    confParser.set(section, 'init2', "AT+CGDCONT=1,\"IP\",\"" + config['apn'] + "\"")
    confParser.set(section, 'password', config['password'])
    confParser.set(section, 'username', config['username'])
    file = open(confFile, 'w')
    confParser.write(file)
    return True

def moveWvDialConfig(d, confFile, confFileTmp):
    if subprocess.call("sudo cp " + confFileTmp + " " + confFile, shell = True):
        d.msgbox("Не удалось сохранить параметры подключения")
        return True
    else:
        d.msgbox("Параметры подключения успешно сохранены")
        subprocess.call("sudo touch /linuxcash/cash/3g-support/configured.flag", shell = True)
        return False

def getDeviceName(port):
    process = subprocess.Popen(["udevadm", "info", "--query=env", "--name=" + port], stdout = subprocess.PIPE, stderr=subprocess.PIPE)
    (stdout, stderr) = process.communicate()
    name = ""
    matches = re.findall("ID_MODEL_FROM_DATABASE=(.*)", stdout)
    if len(matches):
        name = matches[0] + " "
    matches = re.findall("ID_VENDOR_FROM_DATABASE=(.*)", stdout)
    if len(matches):
        name = name + matches[0]
    return name

def searchModems():
    """ Поиск подключенных модемов"""
    process = subprocess.Popen(["wvdialconf", "/tmp/wvdialconf.conf"], stdout = subprocess.PIPE, stderr=subprocess.PIPE)
    (stdout, stderr) = process.communicate()
    lines = re.split("\n", stderr)
    devices = {}
    if os.path.exists('/dev/artix_modem'):
        devices['artix_modem'] = {'baudrate' : "9600", 'descriptor' : "По-умолчанию"}
    for line in lines:
        matches = re.findall("^(.*)<Info>: Speed (.*); init", line)
        if len(matches):
            port, baud = matches[0]
            devices[port] = {'baudrate' : baud, 'descriptor' : getDeviceName(port)}
    return devices

def testConnection(confFile):
    wvdial = subprocess.Popen(["sudo", "wvdial", "-C", confFile], stdout = subprocess.PIPE, stderr=subprocess.PIPE)
    i = 0
    status = False
    try:
        while i < 10 and not wvdial.returncode:
            i = i + 1
            time.sleep(1)
            ifconfig = subprocess.Popen(["ifconfig", "ppp0"], stdout = subprocess.PIPE, stderr=subprocess.PIPE)
            if not ifconfig.wait():
                status = True
                break
    finally:
        if not status:
            wvdial.terminate()
            wvdial = None
    return (status, wvdial)

def chooseProviderDialog(d):
    """Диалог выбора провайдера"""
    items = []
    mark = 1
    providers_keys = providers.keys()
    providers_keys.sort()
    for provider in providers_keys:
        items.append((provider, providers[provider]['title'], mark))
        mark = 0
    return d.radiolist(
        title = "Шаг 1: Выбор провайдера",
        text = "Выберите оператора сотовой связи:",
        height=11, width=45,
        choices = items,
        ok_label=DIALOG_OK_LABEL,
        extra_button="enable", extra_label=DIALOG_EXTRA_LABEL,
        cancel_label=DIALOG_CANCEL_LABEL)

def choosePortDialog(d, devices):
    """Диалог выбора модема"""
    items = []
    mark = 1
    for device in devices.keys():
        items.append((device, devices[device]['descriptor'], mark))
        mark = 0
    return d.radiolist(
        title = "Шаг 2: Выбор устройства",
        text = "Выберите порт:",
        height=11, width=100,
        choices = items,
        ok_label=DIALOG_OK_LABEL,
        extra_button="enable", extra_label=DIALOG_EXTRA_LABEL,
        cancel_label=DIALOG_CANCEL_LABEL)
    
def chooseBaudrateDialog(d, baudrate):
    """Выбор скорости интерфейса"""
    items = []
    insert = True
    mark = 0
    for brate in baudrates:
        if insert and int(baudrate) <= int(brate):
            insert = False
            mark = 1
            if int(baudrate) < int(brate):
                items.append((baudrate, "", mark))
                mark = 0
        items.append((brate, "", mark))
        mark = 0

    return d.radiolist(
        title = "Шаг 3: Выбор скорости устройства",
        text = "Выберите скорость:",
        height=11, width=45,
        choices = items,
        ok_label=DIALOG_OK_LABEL,
        extra_button="enable", extra_label=DIALOG_EXTRA_LABEL,
        cancel_label=DIALOG_CANCEL_LABEL)
  
def testConnectionDialog(d, confFile):
    """Диалог тестового подключения"""
    d.infobox("Подождите. Выполняется тестовое подключение.", height = 6)
    try:
        (status, wvdialProcess) = testConnection(confFile)
        if status:
            d.msgbox("Подключение произведено успешно.", height = 7)
        else:
            d.msgbox("Не удалось подключиться.", height = 7)
        return status, wvdialProcess
    except KeyboardInterrupt:
        d.msgbox("Проверка подключения была прервана", height = 7)
        return False, None
    
  
def pingDialog(d, ip, count):
    """Диалог проверки соединения"""
    (code, tag) = d.form(title = "Проверка соединения",
                         text = "Введите параметры для проверки командой ping",
                         height=10, width=54, form_height=2,
                         fields = [(u"Адрес:", ip, 40),
                                   (u"Кол-во пакетов:", count, 40)])
    if code == d.DIALOG_OK:
        try:
            d.infobox("Подождите. Идет проверка соединения", height = 7)
            ping = subprocess.Popen(['ping', tag[0], '-c', tag[1]], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            ping.wait()
            (stdout, stderr) = ping.communicate()
            if len(stdout) > 0:
                d.scrollbox(stdout)
            elif len(stderr) > 0:
                d.scrollbox(stderr)
            else:
                d.msgbox("Ошибка проверки соединения", height = 6)
        except KeyboardInterrupt:
            d.msgbox("Проверка соединения была прервана", height = 7)
    else:
        return ip, count
    return tag[0], tag[1]