#!/usr/bin/env python
# -*- coding: utf-8 -*-

from wizard import Wizard
import connection_utils
import subprocess

class ConnectionWizard(Wizard):
    def __init__(self, d, wvDialConf, wvDialConfTmp):
        self.d = d
        self.wvDialConf = wvDialConf
        self.wvDialConfTmp = wvDialConfTmp
        self.wvDialProcess = None
        self.config = None
        self.devices = None
        self.pingAddress = ""
        self.pingCount = "4"

        self.currentStep = 0
        self.oldStep = 0
        self.dialogs = [
            {
                'name': 'chooseProvider',
                'function': lambda: self.chooseProvider(),
                'back': None,
                'next': 'searchModems'
            },
            {
                'name': 'searchModems',
                'function': lambda: self.searchModems(),
                'back': 'chooseProvider',
                'next': 'choosePort'
            },
            {
                'name': 'choosePort',
                'function': lambda: self.choosePort(),
                'back': 'chooseProvider',
                'next': 'chooseBaudrate'
            },
            {
                'name': 'chooseBaudrate',
                'function': lambda: self.chooseBaudrate(),
                'back': 'choosePort',
                'next': 'testConnect'
            },
            {
                'name': 'testConnect',
                'function': lambda: self.testConnect(),
                'back': 'chooseBaudrate',
                'next': {
                         'connected' : 'ping',
                         'notConnected' : 'finish'
                         }
            },
            {
                'name': 'ping',
                'function': lambda: self.ping(),
                'back': 'chooseBaudrate',
                'next': 'finish'
            },
            {
                'name': 'finish',
                'function': lambda: self.finish(),
                'back': 'chooseBaudrate',
                'next': None
            }                    
        ]
        self.loadConf()
    
    def loadConf(self):
        self.config = connection_utils.loadWvDialConfig(self.wvDialConfTmp)
        if not self.config:
            self.d.msgbox("Не удалось загрузить файл конфигурации - " + self.wvDialConfTmp)
            self.close()
        
    
    def chooseProvider(self):
        """Выбор оператора""" 
        (code, tag) = connection_utils.chooseProviderDialog(self.d)
        if code == self.d.DIALOG_OK:
            connection_utils.setProvider(self.config, tag)
            self.nextForDialog("chooseProvider");
        else:
            self.close()
   
    def searchModems(self):
        """Поиск модемов"""
        self.d.infobox("Подождите. Программа выполняет поиск устройств.", height=6)
        
        try:
            self.devices = connection_utils.searchModems()
            if len(self.devices):
                self.nextForDialog("searchModems");
            else:
                self.d.msgbox("Устройства не найдены.", height = 5)
                self.backForDialog("searchModems");
        except KeyboardInterrupt:
            self.d.msgbox("Поиск устройств был прерван", height = 7)
            self.backForDialog("searchModems");
 
    def choosePort(self):
        """Выбор устройства"""
        (code, tag) = connection_utils.choosePortDialog(self.d, self.devices)
        if code == self.d.DIALOG_OK:
            self.config['port'] = "/dev/" + tag
            self.config['baudrate'] = self.devices[tag]['baudrate']
            self.nextForDialog("choosePort");
        elif code == self.d.DIALOG_EXTRA:
            self.backForDialog("choosePort");
        else:
            self.close()
    
    def chooseBaudrate(self):
        """Выбор устройства"""
        (code, tag) = connection_utils.chooseBaudrateDialog(self.d, self.config['baudrate'])
        if code == self.d.DIALOG_OK:
            self.config['baudrate'] = tag
            
            """Сохраняем все настройки в tmp файл"""
            if not connection_utils.saveWvDialConfig(self.wvDialConfTmp, self.config):
                self.d.msgbox("Не удалось сохранить файл конфигурации - " + self.wvDialConfTmp)
                self.close()
            
            self.nextForDialog("chooseBaudrate");
        elif code == self.d.DIALOG_EXTRA:
            self.backForDialog("chooseBaudrate");
        else:
            self.close()
   
    def testConnect(self):
        """Тестирование подключения"""
        code = self.d.yesno("Выполнить тестовое подключение?",
                            ok_label=connection_utils.DIALOG_OK_LABEL,
                            extra_button="enable", extra_label=connection_utils.DIALOG_EXTRA_LABEL,
                            cancel_label=connection_utils.DIALOG_SKIP_LABEL,
                            width = 50, height = 7)
        if code == self.d.DIALOG_OK:
            (status, wvdialProcess) = connection_utils.testConnectionDialog(self.d, self.wvDialConfTmp)
            self.wvDialProcess = wvdialProcess
            if status:
                self.nextForDialog("testConnect", "connected");

        elif code == self.d.DIALOG_CANCEL:
            self.nextForDialog("testConnect", "notConnected");
        else:
            self.backForDialog("testConnect");
            
    def ping(self):
        """Проверка соединения"""
        self.pingAddress, self.pingCount = connection_utils.pingDialog(self.d, self.pingAddress, self.pingCount)
        self.wvDialProcess.terminate()
        self.wvDialProcess = None
        self.nextForDialog("ping")

    def finish(self):
        """Сохраниение настроек"""
        code = self.d.yesno("Сохранить параметры подключения?",
                            ok_label=connection_utils.DIALOG_OK_LABEL,
                            extra_button="enable", extra_label=connection_utils.DIALOG_EXTRA_LABEL,
                            cancel_label=connection_utils.DIALOG_CANCEL_LABEL,
                            width = 50, height = 5)
        if code == self.d.DIALOG_OK:
            connection_utils.moveWvDialConfig(self.d, self.wvDialConf, self.wvDialConfTmp)
            self.close()
            
        elif code == self.d.DIALOG_CANCEL:
            self.close()
        else:
            self.backForDialog("testConnect");
       